// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <stdlib.h>

#include "BoolPar.h"

static const char* yep[]  = { "yes", "y", "1", NULL };
static const char* nope[] = { "no",  "n", "0", NULL };

BoolPar::BoolPar( ParTxt& par ) throw ( ParFileException ) : Par( par ) {

  try {

    validate( parameter[ PARMINIMUM ] );
  
    validate( parameter[ PARMAXIMUM ] );
  
    validate( parameter[ PARVALUE ] );

  } catch( ParFileException& pfe ) {
    throw;
  }
  
}
  
void BoolPar::between_limits( const string& str ) const
  throw ( ParFileException, Exception ) {

    try {
      validate( str );
    } catch( ParFileException& pfe ) {
      throw;
    }

}

bool BoolPar::pgetb( void ) const throw ( ParFileException ) {

  size_t ii = 0;
  while( yep[ ii ] )
    if ( parameter[ PARVALUE ] == yep[ ii++ ] )
      return true;

  ii = 0;
  while( nope[ ii ] ) {
    if ( parameter[ PARVALUE ] == nope[ ii++ ] ) {
      return false;
    }
  }

  throw ParFileException( "BoolPar::pgetb( ) : Unable to get the value "
			  "for parameter " + parameter[ PARNAME ] );

}

void BoolPar::set_val( const string& str ) throw ( ParFileException,
						   Exception ) {

  try {
    validate( str );
    parameter[ PARVALUE ] = str;
  } catch( ParFileException& pfe ) {
    throw;
  } catch( Exception& e ) {
    throw;
  }

}

void BoolPar::validate( const string& str ) const throw ( ParFileException ) {

  if ( Par::is_indirrect( str ) )
    return;

  if ( "" == str )
    return;

  size_t ii = 0;
  while( yep[ ii ] )
    if ( str == yep[ ii++ ] )
      return;

  ii = 0;
  while( nope[ ii ] )
    if ( str == nope[ ii++ ] )
      return;

  throw ParFileException( "BoolPar::validate( " + str + " ) : Ivalid arg\n" );

}
