#ifndef NameAttributeValue_H
#define NameAttributeValue_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <string>

using namespace std;

/**
 * Parse the name, attribute (if any) and value from a string.
 *
 * The class parses a string of the form : 
 *       name.attribute=value, name+, name-, --name
 *  for the name `name', attribute `attribute' and value `value'.
 */
class NameAttributeValue {

  /**
   * Output operator. This operator outputs the contents of the object.
   */
  friend ostream& operator << ( ostream& os, NameAttributeValue& a ) {
    a.print( os );
    return os;
  }

  /**
   * Output operator. This operator outputs the contents of the object.
   */
  friend ostream& operator << ( ostream& os, NameAttributeValue* a ) {
    a->print( os );
    return os;
  }

public:

  /// Destructor
  virtual ~NameAttributeValue( ) { }

  /// Default constructor, all members are initialize to an empty string.
  NameAttributeValue( ) : name( "" ), attribute( "" ), value( "" ) { }

  /**
   * Parse a string, str, into name, attribute and value.
   * Equivalent to calling the default constructor then initialize_NameAttributeValue.
   */
  NameAttributeValue( const char* str ) { init_NameAttributeValue( str ); }

  /// return the attribute of the object
  string get_attribute( void ) { return attribute; }

  /// return the name of the object
  string get_name( void ) const { return name; }

  /// return the pointer to the value of the object
  string get_value( void ) const { return value; }

  /// Set the name, attribute and value
  void init_NameAttributeValue( const char* str );

  /// Print the content of the object to a stream.  
  virtual void print( ostream& os=cerr );

  /// set the attribute of the object.
  void  set_attribute( const char* a ) { attribute = a; }

  /// set the name of the object.
  void set_name( const char* n ) { name = n; }

  /// set the value of the object.
  void set_value( const char* v ) { value = v; }

protected:

  string name;

  string attribute;

  string value;

private:

  bool check_boolean( char* str );

  void seperate_keyword_attribute( char* );

  void seperate_keyword_value( char* );

  NameAttributeValue( const NameAttributeValue& s );
  NameAttributeValue& operator = ( const NameAttributeValue& rhs );

};

#endif
