#ifndef ParFilename_H
#define ParFilename_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <string>
#include <vector>

#include "ParFileException.h"

using namespace std;

/**
 * Find the parameter file.
 * If PFILE=foo is set at the command line then the file `foo' shall be
 * the file to be opened.  PFILE will overide even if a specific file
 * to the function (3rd argument) Parfile is set.
 * The class loops over the PFILES, UPARM environment variables to determine
 * then the .par and extension to locate the parameter file.
 */
class ParFilename {

  /**
   * This output operator outputs the contents of the object.
   */
  friend ostream& operator << ( ostream& os, ParFilename& a ) {
    a.print( os );
    return os;
  }

  /**
   * This output operator outputs the contents of the object.
   */
  friend ostream& operator << ( ostream& os, ParFilename* a ) {
    os << *a;
    return os;
  }

  enum ParFileDelimit { pfiles_delimit = ':', dir_delimit = '/' };

public:

  /**
   * destructor
   */
  ~ParFilename( void ) { }

  /**
   * default constructor
   */
  ParFilename( ) { }

  /**
   * constructor
   */
  ParFilename( int argc, char** argv, const char* file = NULL )
    throw ( ParFileException );

  /**
   * Get the parameter filename
   */
  string get_filename( void ) { return filename; }

  /**
   * Initialize the parameter
   */
  void init_ParFilename( int argc, char** argv,
			 const char* file = NULL ) throw ( ParFileException );

  static vector< string > parse_string( string& str, char delimit );

  void print( ostream& os ) { 
    os << filename;
  }

private:

  /**
   * The parameter filename
   */
  string            filename;    
     
  int accessible( const char* str ) throw ( ParFileException );

  void copy_pfile( const char* str ) const throw ( ParFileException );

  /**
   *
   * One of the environment variables (PFILES, UPARM) must be defined.
   * An environment variable PFILES or UPARM must be defined to let the
   * lib know where to read the parameter file from.
   *
   */
  void check_environment_variables( ) throw ( ParFileException );

  /**
   * Find the parameter file.
   * Check if the given name is legit.
   * Loop thru the parameter variables and the possible extensions
   * to find the parameter file.
   */
  int find_pfile( const char* name ) throw ( ParFileException );

  const char* get_basename( const char* name ) const ;

  /**
   * Declare but purposely do not define
   */
  ParFilename( const ParFilename& pf );

  /**
   * Declare but purposely do not define
   */
  ParFilename& operator = ( const ParFilename& pf );

};

#define PFILE                   "pfile="

#endif
