#ifndef PARAMETERS_H
#define PARAMETERS_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <map>
#include <iostream>

#include <ParFile.h>

using namespace std;

class Parameters {

  friend ostream& operator << ( ostream& os, Parameters& a ) {
    a.print( os );
    return os;
  }
  //: Output operator.
  // This operator outputs the contents of the object.

  friend ostream& operator << ( ostream& os, Parameters* a ) {
    a->print( os );
    return os;
  }
  //: Output operator.
  // This operator outputs the contents of the object.
  
public:

  virtual ~Parameters( ) { }

  Parameters( int argc, char* argv[], const char* fname=NULL,
	      const char* boolpars[]=0, const char* doublepars[]=0,
	      const char* intpars[]=0, const char* stringpars[]=0 )
    throw ( ParFileException );

  void init_Parameters( int argc, char* argv[],const char* fname=NULL,
			const char* boolpars[]=0, const char* doublepars[]=0,
			const char* intpars[]=0, const char* stringpars[]=0 )
    throw ( ParFileException );

  virtual void print( ostream& os=cout, const char* prefix="#: " ) const;

  bool   pgetb( const char* ) const throw ( ParFileException );

  int    pgeti( const char* ) const throw ( ParFileException );

  double pgetd( const char* ) const throw ( ParFileException );

  string pgetstring( const char* ) const throw ( ParFileException );

private:

  map< string, bool,   less< string > > bool_parameters;

  map< string, double, less< string > > double_parameters;
  
  map< string, int,    less< string > > int_parameters;

  map< string, string, less< string > > string_parameters;

  void init_boolpars( ParFile& pf, const char* boolpars[] )
    throw ( ParFileException );

  void init_doublepars( ParFile& pf, const char* doublepars[] )
    throw ( ParFileException );

  void init_intpars( ParFile& pf, const char* intpars[] )
    throw ( ParFileException );

  void init_stringpars( ParFile& pf, const char* stringpars[] )
    throw ( ParFileException );

};

#endif
